#include <stdio.h>
#include <math.h>
#include "edges.h"
#include "pgm.h"

int *temp_map[ORIENTATIONS];
/*
main(int argc, char* argv[])
{
  int width, height, max;
  int i, j;
  int size;
  unsigned char *edge_map[ORIENTATIONS];
  unsigned char *image;

  max=read_pgm(argv[1], &image, &width, &height);
  size=width*height;

  Edges(width, height, size, edge_map, image);

  write_pgm("edge000.pgm", edge_map[0], width, height, max);
  write_pgm("edge030.pgm", edge_map[1], width, height, max);
  write_pgm("edge045.pgm", edge_map[2], width, height, max);
  write_pgm("edge060.pgm", edge_map[3], width, height, max);
  write_pgm("edge090.pgm", edge_map[4], width, height, max);
  write_pgm("edge120.pgm", edge_map[5], width, height, max);
  write_pgm("edge135.pgm", edge_map[6], width, height, max);
  write_pgm("edge150.pgm", edge_map[7], width, height, max);

}

*/
void Edges(int width, int height, int size, unsigned char **edge_map,
		unsigned char *image)
{
  int i, j, edge;
  int radius=2;


  for (i=0; i<ORIENTATIONS; i++)
  {
    temp_map[i]=(int *) calloc(4*size, sizeof(int));
    edge_map[i]=(unsigned char *) calloc(4*size, 1);
  }

  for (edge=0; edge<ORIENTATIONS; edge++)
  {
    for (i=radius; i<height-radius; i++)
    {
      for (j=radius; j<width-radius; j++)
      {
        temp_map[edge][i*width+j]=
				(int)abs(Sobel(i, j, edge, width, image));
      }
    }
  }

  MultipleNormalize(size, edge_map);

}


float Sobel(int i, int j, int hierarchy, int width, unsigned char *image)
{
  float temp;

  static int degrees_000[25] = {-1, -1, -1, -1, -1,
				-1, -1, -1, -1, -1,
				 2,  2,  2,  2,  2,
				-1, -1, -1, -1, -1,
				-1, -1, -1, -1, -1 };

  static int degrees_030[25] = {-1, -1, -1, -1, -1,
				-1, -1, -1, -1,  2,
				-1,  2,  2,  2, -1,
				 2, -1, -1, -1, -1,
				-1, -1, -1, -1, -1 };

  static int degrees_045[25] = {-1, -1, -1, -1,  2,
				-1, -1, -1,  2, -1,
				-1, -1,  2, -1, -1,
				-1,  2, -1, -1, -1,
				 2, -1, -1, -1, -1 };

  static int degrees_060[25] = {-1, -1, -1,  2, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1,  2, -1, -1, -1 };

  static int degrees_090[25] = {-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1 };

  static int degrees_120[25] = {-1,  2, -1, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1, -1,  2, -1 };
 
 static int degrees_135[25] = { 2, -1, -1, -1, -1,
				-1,  2, -1, -1, -1,
				-1, -1,  2, -1, -1,
				-1, -1, -1,  2, -1,
				-1, -1, -1, -1,  2 };

  static int degrees_150[25] = {-1, -1, -1, -1, -1,
				 2, -1, -1, -1, -1,
				-1,  2,  2,  2, -1,
				-1, -1, -1, -1,  2,
				-1, -1, -1, -1, -1 };

  if (ORIENTATIONS==8)
    switch(hierarchy)
    {
      case 0: return Grad(i, j, width, degrees_000, image);
      case 1: return Grad(i, j, width, degrees_030, image);
      case 2: return Grad(i, j, width, degrees_045, image);
      case 3: return Grad(i, j, width, degrees_060, image);
      case 4: return Grad(i, j, width, degrees_090, image);
      case 5: return Grad(i, j, width, degrees_120, image);
      case 6: return Grad(i, j, width, degrees_135, image);
      case 7: return Grad(i, j, width, degrees_150, image);
    }
  else if (ORIENTATIONS==4)
    switch(hierarchy)
    {
      case 0: return Grad(i, j, width, degrees_000, image);
      case 1: return Grad(i, j, width, degrees_045, image);
      case 2: return Grad(i, j, width, degrees_090, image);
      case 3: return Grad(i, j, width, degrees_135, image);
    }     
}



float Grad(int i, int j, int width, int val[24], unsigned char *image)
{
  return(val[0]  * image[(i-2) * width + (j-2)] +
         val[1]  * image[(i-2) * width + (j-1)] +
         val[2]  * image[(i-2) * width +  (j) ] +
         val[3]  * image[(i-2) * width + (j+1)] +
         val[4]  * image[(i-2) * width + (j+2)] +
         val[5]  * image[(i-1) * width + (j-2)] +
         val[6]  * image[(i-1) * width + (j-1)] +
         val[7]  * image[(i-1) * width +  (j) ] +
         val[8]  * image[(i-1) * width + (j+1)] +
         val[9]  * image[(i-1) * width + (j+2)] +
         val[10] * image[ (i)  * width + (j-2)] +
         val[11] * image[ (i)  * width + (j-1)] +
         val[12] * image[ (i)  * width + (j)] +	
         val[13] * image[ (i)  * width + (j+1)] +
         val[14] * image[ (i)  * width + (j+2)] +
         val[15] * image[(i+1) * width + (j-2)] +
         val[16] * image[(i+1) * width + (j-1)] +
         val[17] * image[(i+1) * width +  (j) ] +
         val[18] * image[(i+1) * width + (j+1)] +
         val[19] * image[(i+1) * width + (j+2)] +
         val[20] * image[(i+2) * width + (j-2)] +
         val[21] * image[(i+2) * width + (j-1)] +
         val[22] * image[(i+2) * width +  (j) ] +
         val[23] * image[(i+2) * width + (j+1)] +
         val[24] * image[(i+2) * width + (j+2)] );
}




void MultipleNormalize(int size, unsigned char **e_map)
{
  int i;
  float currMax, maxVal = 0.0;
  
  for(i=0; i< ORIENTATIONS; i++)
  {
    currMax = Maximum(i, size);
    if (currMax > maxVal)
       maxVal=currMax;
  }

  for(i=0; i< ORIENTATIONS; i++)

    Normalize(i, maxVal, size, e_map);

  return;
}  



void Normalize (int edge, int maxValue, int size, unsigned char **e_map)
{
  int i;
  float multFac;

  multFac=255.0/maxValue;

  for(i=0; i<size; i++)
    e_map[edge][i]=(unsigned char)(temp_map[edge][i]*multFac);
}



float Maximum(int edge, int size)
{
  int i;
  float max=0;

  for (i=0; i<size; i++)
  {
    if (temp_map[edge][i]>max)
      max=temp_map[edge][i];
  }

  return max;
}
