#include <stdio.h>
#include <math.h>
#include "utils.h"
#include "edges.h"

/*
void Calc_Gaus(int size, long sum, double **matrix)
{
   int i, j;
   long *x;
   
   sum=0;

   x = (long *) calloc( (int) size, (int) sizeof(long) );
   printf( "%2d x %2d Gaussian based on: [", size, size );
   for( i=0; i<size; i++ ){
      x[i] = Binomial( (size-1), i );
      printf( "%4ld ", x[i] );
   }
   printf( "  ]\n" );


   /* Build matrix based on (X-transpose * X) 
   matrix = (double **) calloc( size, sizeof( double *) );
   for (i=0; i<size; i++)
     matrix[i]=(double *) calloc(size, sizeof(double));

   for( i=0; i<size; i++ )
   {
      printf( "\n" );
      for( j=0; j<size; j++ )
      {
         matrix[i][j] = (float) x[i]*x[j];
         sum += (long) matrix[i][j];
         printf( "%6ld ", (long) matrix[i][j] );

      }
   }
   printf( "\n\n" );

   /* Normalize matrix to floats that add to one 
   for( i=0; i<size; i++ )
   {
      for( j=0; j<size; j++ )
      {
         matrix[i][j] /= (float) sum;
      }
   }

   return;
}


void center_surround(int c_rad, int s_rad, double **c_gaus, double **s_gaus)
{
  long c_total, s_total;
  int i, j, size;
  double factor;

  Calc_Gaus(c_rad*2+1, c_total, c_gaus);
  
  size=s_rad*2+1;
  Calc_Gaus(size, s_total, s_gaus);

  factor=(double)c_total/(double)s_total;
  for (i=0; i<size; i++)
    for (j=0; j<size; j++)
      s_gaus[i][j]*=factor;
}
*/


void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;
  FILE *f_gaus;

  f_gaus=fopen("gaus1", "w");


  fprintf(f_gaus, "rad=%d\n", rad);

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/2.0);
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
  {
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
      fprintf(f_gaus, "%4d", (int)(gaus[i+rad][j+rad]*100000));
    }
    fprintf(f_gaus, "\n");
  }
}


void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.; 
  denom=ORIENTATIONS*ORIENTATIONS;
  if (ORIENTATIONS%2)
    denom=denom-1;

  for (i=1; i<=ORIENTATIONS/2; i++)
  {
    w[i]=(float)-4*i/denom;
    w[ORIENTATIONS-i]=w[i];
  }
}

    

double Convolve(int x, int y, int width, int rad, double **gaus,
		unsigned char *c_map)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
	temp+=(double) c_map[(y-i)*width+(x-j)] * 
		gaus[i+rad][j+rad];

  return temp;
}


void Combine_Maps(double **dbl_map, int size, unsigned char *c_map)
{
  int i, j;
  double max=0.;

  for(i=0; i<size; i++)
  {
    for (j=0; j<ORIENTATIONS; j++)
      if (dbl_map[j][i]>max)
	max=dbl_map[j][i];
    c_map[i]=(char)max;
    max=0.;
  }

  return;
}



double Mult_Norm_dbl(double **array, int size)
{
  int i;
  double t_max, max=0;

  for(i=0; i<ORIENTATIONS; i++)
    if ((t_max=Max_dbl(array[i], size))>max)
      max=t_max;

  for(i=0; i<ORIENTATIONS; i++)
    Norm_dbl(array[i], max, size);

  return max;
}




void Norm_dbl(double *array, double max, int size)
{
  int i;
  double mult_fac;
  FILE *fa, *fb;


  fa=fopen("temp2a", "w");
  fb=fopen("temp2b", "w");

  if (max>0)
  {
    mult_fac=255./max;
    printf("norm_dbl\nmax=%g factor=%g\n", max, mult_fac);
    for(i=0; i<size; i++)
    {
      fprintf(fa, "%d %g\n", i, array[i]);
      array[i]=array[i]*mult_fac;
      fprintf(fb, "%d %g\n", i, array[i]);
    }
  }
  else
    for(i=0; i<size; i++)
      array[i]=0;
  fclose(fa);
  fclose(fb);

  return;
}
    


void Norm_char(char* map, int size)
{
  int i, j, max;
  float multFac;

  max=Max_char(map, size);

  if(max>0)
  {
    multFac=255.0/max;

    for(i=0; i<size; i++)
      map[i]=map[i]*multFac;
  }
}




int Max_int(int *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}

double Max_dbl(double *map, int size)
{
  int i;
  double max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}


int Max_char(char *map, int size)
{
  int i;
  int max=0;
  
  for (i=0; i<size; i++)
  {
    if (map[i]>max)
      max=map[i];
  }

  return max;

}

int max(int a, int b)
{
  if (a>b) return a;
  return b;
}

long FallingFactorial( int a, int b )
{
   int i;
   long result = 1;

   for( i=b; i<=a; i++ )
      result *= i;

   return result;

} /* FallingFactorial() */

long Binomial( int a, int b )
{
   int  maxDenominator = max( b, (a-b) );

   return( FallingFactorial( a, (maxDenominator+1) ) /
           FallingFactorial( (a-maxDenominator), 2 ) );

} /* Binomial() */



int Write_dbl_pgm(char *file_name, double *map, int width, 
		  int height, int max)
{
  unsigned char c_map[height*width];
  int i;
  
  for(i=0; i<height*width; i++)
  {
    if (map[i]>0.)
      c_map[i]=(unsigned char) map[i];
    else
      c_map[i]=0;
  }

  write_pgm(file_name, c_map, width, height, max);
}



void dbl_to_char(unsigned char *map, double *dbl_map, int size)
{
  int i;

  for (i=0; i<size; i++)
  {
    if (dbl_map[i]>0.)
      map[i]=(unsigned char) dbl_map[i];
    else
      map[i]=0;
  }
	
}


void num_to_str(int num, char *str)
{
  int i=0, num2;

  num2=num;
  while(num2)
  {
    num2=num2/10;
    i++;
  }

  str[i--]=0;
  num2=num;
  while(num2)
  {
    str[i--]='0'+num2%10;
    num2=num2/10;
  }
}

