#include<stdio.h>
#include<stdlib.h>
#include "pgm.h"
#include "edges.h"
#include "contrast.h"
#include "test.h"
#include "globals.h"

main(int argc, char *argv[])
{
  int i, j;
  int level_size[Levels];

  unsigned char *image;	/* for read pgm */
  int width, height, max;
  int size;
  int elsize, c_rad;

  unsigned char **edge_map;
  unsigned char *inten_map[2];

  unsigned char ***edge_tree;
  unsigned char ***inten_tree;

  double **dbl_map;
  unsigned char ***edge_data;
  unsigned char ***inten_data;
  unsigned char *c_map;
  double max_con, t_max;
  int max_ind;

  char f_name[20];

  read_data_file(argv[1]);
 
  printf("\n%s\n", Image_Name);
  
  /* read pgm file */
  max = read_pgm(Image_Name, &image, &width, &height);
  size=width*height;
  
  dbl_map = (double **) calloc(Levels, sizeof(double *));

  level_size[Levels-1]=width;
  for (i=Levels-2; i>=0; i--)
    level_size[i]=level_size[i+1]/2;

  if (I_Contrast)
  {
    inten_data = (unsigned char ***) calloc(Levels, sizeof(unsigned char **));
    for (i=0; i<Levels; i++)
      inten_data[i] = (unsigned char **) calloc(2, sizeof(unsigned char *));

    inten_tree = (unsigned char ***) calloc(2, sizeof(unsigned char **));
    inten_tree[0] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));
    inten_tree[1] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));
   
    inten_map[0] = (unsigned char *) malloc(size);
    inten_map[1] = (unsigned char *) malloc(size);


    /* set up intensity maps */
    norm_char(inten_map[0], image, size);
    negate(inten_map[1], inten_map[0], size);

    /* make intensity trees */
    tree(inten_map[0], inten_tree[0], level_size);
    tree(inten_map[1], inten_tree[1], level_size);

    /* find contrast for each level of resolution */
    for (i=0; i<Levels; i++)
    {
      inten_data[i][0]=inten_tree[0][i];
      inten_data[i][1]=inten_tree[1][i];
    }
  }

  if (O_Contrast)
  {
    edge_map = (unsigned char **) 
			calloc(Num_Orientations, sizeof(unsigned char *));

    edge_data = (unsigned char ***) calloc(Levels, sizeof(unsigned char **));
    for (i=0; i<Levels; i++)
      edge_data[i] = (unsigned char **)
			calloc(Num_Orientations, sizeof(unsigned char *));

    edge_tree = (unsigned char ***)
			calloc(Num_Orientations, sizeof(unsigned char **));
    for(i=0; i<Num_Orientations; i++)
      edge_tree[i] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));

    /* find edges (edge_maps) */
    Edges(width, height, size, edge_map, image);

    /* make orientation trees */
    for (i=0; i<Num_Orientations; i++)
      tree(edge_map[i], edge_tree[i], level_size);

    /* find contrast for each level of resolution */
    for (i=0; i<Levels; i++)
    {
      for (j=0; j<Num_Orientations; j++)
        edge_data[i][j]=edge_tree[j][i];
    } 
  }

  printf("contrast\n");

  i=1;
  max_con=0;
  for (c_rad=Min_Radius; c_rad<=Max_Radius; c_rad++)
  {
    contrast(inten_data[i], edge_data[i], level_size[i],
		&(dbl_map[c_rad-Min_Radius]), c_rad, &t_max, 1);

    printf("iter: %d\n", c_rad);

    if (t_max>max_con)
    {
      max_con=t_max;
      max_ind=c_rad-Min_Radius;
      printf("max_ind=%d\n", max_ind);
    }

  }


  /* convert double map to chars for saving */
  c_map = (unsigned char *) malloc(level_size[i]*level_size[i]);
  d_to_c(dbl_map[max_ind], c_map, level_size[i]*level_size[i]);

  /* write results to pgm files */
  strncpy(f_name, Image_Name, strlen(Image_Name)-3);
  f_name[strlen(Image_Name)-3]=0;
  sprintf(f_name, "%sc.pgm", f_name);
  write_pgm(f_name, c_map, level_size[i], level_size[i], max);
 
}

void norm_char(unsigned char *to, unsigned char *from, int size)
{
  int i, max=0; 
  double factor;

  for(i=0; i<size; i++)
    if (from[i]>max) max=from[i];

  factor=255.0/max;
  for(i=0; i<size; i++)
    to[i]=from[i]*factor;
}

void negate(unsigned char *to, unsigned char *from, int size)
{
  int j;

  for(j=0; j<size; j++)
    to[j]=255-from[j];
}
